import sys
import os
import json

import xmc

if __name__ == "__main__":

    if len(sys.argv) == 2:
        parametersPath = str(sys.argv[1])  # set path to the parameters
    else:
        parametersPath = (
            "../problem_settings/parameters_xmc_test_mlmc_Kratos_poisson_2d_with_combined_power_sums.json"
        )

    # read parameters
    with open(parametersPath, "r") as parameter_file:
        parameters = json.load(parameter_file)

    # add path of the problem folder to python path
    problem_id = parameters["solverWrapperInputDictionary"]["problemId"]

    # SampleGenerator
    samplerInputDictionary = parameters["samplerInputDictionary"]
    samplerInputDictionary["randomGeneratorInputDictionary"] = parameters[
        "randomGeneratorInputDictionary"
    ]
    samplerInputDictionary["solverWrapperInputDictionary"] = parameters[
        "solverWrapperInputDictionary"
    ]

    # MonteCarloIndex Constructor
    monteCarloIndexInputDictionary = parameters["monteCarloIndexInputDictionary"]
    monteCarloIndexInputDictionary["samplerInputDictionary"] = samplerInputDictionary

    # Moment Estimators

    # qoi estimators
    if "qoiEstimatorInputDictionary" not in monteCarloIndexInputDictionary.keys():
        numberQoI = parameters["solverWrapperInputDictionary"]["numberQoI"]
        monteCarloIndexInputDictionary["qoiEstimator"] = [
            monteCarloIndexInputDictionary["qoiEstimator"][0] for _ in range(numberQoI)
        ]
        monteCarloIndexInputDictionary["qoiEstimatorInputDictionary"] = [
            parameters["qoiEstimatorInputDictionary"]
        ] * numberQoI
    # combined estimators
    if "combinedEstimator" in monteCarloIndexInputDictionary.keys():
        numberCombinedQoI = parameters["solverWrapperInputDictionary"]["numberCombinedQoi"]
        monteCarloIndexInputDictionary["combinedEstimator"] = [
            monteCarloIndexInputDictionary["combinedEstimator"][0]
            for _ in range(numberCombinedQoI)
        ]
        monteCarloIndexInputDictionary["combinedEstimatorInputDictionary"] = [
            parameters["combinedEstimatorInputDictionary"]
        ] * numberCombinedQoI
    # cost estimator
    monteCarloIndexInputDictionary["costEstimatorInputDictionary"] = parameters[
        "costEstimatorInputDictionary"
    ]

    # MonoCriterion
    criteriaArray = []
    criteriaInputs = []
    for monoCriterion in parameters["monoCriteriaInpuctDictionary"]:
        criteriaArray.append(
            xmc.monoCriterion.MonoCriterion(
                parameters["monoCriteriaInpuctDictionary"][monoCriterion]["criteria"],
                parameters["monoCriteriaInpuctDictionary"][monoCriterion]["tolerance"],
            )
        )
        criteriaInputs.append(
            [parameters["monoCriteriaInpuctDictionary"][monoCriterion]["input"]]
        )

    # MultiCriterion
    multiCriterionInputDictionary = parameters["multiCriterionInputDictionary"]
    multiCriterionInputDictionary["criteria"] = criteriaArray
    multiCriterionInputDictionary["inputsForCriterion"] = criteriaInputs
    criterion = xmc.multiCriterion.MultiCriterion(**multiCriterionInputDictionary)

    # ErrorEstimator
    MSEErrorEstimator = xmc.errorEstimator.ErrorEstimator(
        **parameters["errorEstimatorInputDictionary"]
    )

    # HierarchyOptimiser
    hierarchyCostOptimiser = xmc.hierarchyOptimiser.HierarchyOptimiser(
        **parameters["hierarchyOptimiserInputDictionary"]
    )

    # EstimationAssembler
    if "expectationAssembler" in parameters["estimationAssemblerInputDictionary"].keys():
        expectationAssembler = xmc.estimationAssembler.EstimationAssembler(
            **parameters["estimationAssemblerInputDictionary"]["expectationAssembler"]
        )
    if (
        "discretizationErrorAssembler"
        in parameters["estimationAssemblerInputDictionary"].keys()
    ):
        discretizationErrorAssembler = xmc.estimationAssembler.EstimationAssembler(
            **parameters["estimationAssemblerInputDictionary"]["discretizationErrorAssembler"]
        )
    if "varianceAssembler" in parameters["estimationAssemblerInputDictionary"].keys():
        varianceAssembler = xmc.estimationAssembler.EstimationAssembler(
            **parameters["estimationAssemblerInputDictionary"]["varianceAssembler"]
        )

    # MonteCarloSampler
    monteCarloSamplerInputDictionary = parameters["monteCarloSamplerInputDictionary"]
    monteCarloSamplerInputDictionary[
        "indexConstructorDictionary"
    ] = monteCarloIndexInputDictionary
    monteCarloSamplerInputDictionary["assemblers"] = [
        expectationAssembler,
        discretizationErrorAssembler,
        varianceAssembler,
    ]
    monteCarloSamplerInputDictionary["errorEstimators"] = [MSEErrorEstimator]
    mcSampler = xmc.monteCarloSampler.MonteCarloSampler(**monteCarloSamplerInputDictionary)

    # XMCAlgorithm
    XMCAlgorithmInputDictionary = parameters["XMCAlgorithmInputDictionary"]
    XMCAlgorithmInputDictionary["monteCarloSampler"] = mcSampler
    XMCAlgorithmInputDictionary["hierarchyOptimiser"] = hierarchyCostOptimiser
    XMCAlgorithmInputDictionary["stoppingCriterion"] = criterion

    algo = xmc.XMCAlgorithm(**XMCAlgorithmInputDictionary)

    if parameters["solverWrapperInputDictionary"]["asynchronous"] is True:
        algo.runAsynchronousXMC()
    else:
        algo.runXMC()